//-----------------------------------------------------------------------------
//
//			      XMM Science Analysis System (SAS)
//
//-----------------------------------------------------------------------------
//
//	EnergyCombinator.cc --- implementation of EnergyCombinator class
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		UL, 1999-11-19: 1.0.0: - framework (to be taken over by Michel
//						    Lortholary/Jean Ballet)
//		UL, 2000-05-22: 1.1.0: - brought in line with cal-3.71
//
//	AUTHOR:
//		U. Lammers, Fri Nov 19 15:35:09 MET 1999
//
//	REFERENCES:
//
//	$Header: /data/xmm02/cvsadmin/soc_repos/sas/packages/calmosalgo/src/EnergyCombinator.cc,v 1.3 2000/05/22 07:35:46 ulammers Exp $
//

#include "EnergyCombinator.h"
#include "errstr.h"				//	SAS error package
#include "common.h"				//	from utils
#include "EventAttributes.h"


CalReal32Vector &EnergyCombinator::combine(const CalReal32Vector &energye1,
					   const CalInt16Vector &energye2,
					   const CalInt8Vector &pattern,
                                           CalReal32Vector &pha     // out
				           				   
                                           ) 
{
	PROC

        bool withLocbkg = false ;
        bool fulldata = false ;
        
        // Dummy arguments
        CalReal32Vector locbkg;
        CalInt16Vector energye3;
        CalInt16Vector energye4;
        CalInt8Vector   peripix;
        CalInt32Vector   flag;

        AlgoCombine( energye1,energye2,pattern, pha, locbkg, energye3,
                     energye4, peripix, flag, withLocbkg, fulldata );
           
	RETURN pha;
} // EnergyCombinator::combine()


CalReal32Vector &EnergyCombinator::combine(const CalReal32Vector &energye1,
					   const CalInt16Vector &energye2,
					   const CalInt8Vector &pattern,
                                           CalReal32Vector &pha,    // out
				           
                                           const CalReal32Vector &locbkg
					   
                                           ) 
{
	PROC

        bool withLocbkg = true ;
        bool fulldata = false ;
                
        // Dummy arguments
        CalInt16Vector energye3;
        CalInt16Vector energye4;
        CalInt8Vector   peripix;
        CalInt32Vector   flag;

        AlgoCombine( energye1,energye2,pattern, pha, locbkg, energye3,
                     energye4, peripix, flag, withLocbkg, fulldata );

	RETURN pha;
} // EnergyCombinator::combine()


CalReal32Vector &EnergyCombinator::combine(const CalReal32Vector &energye1,
					   const CalInt16Vector &energye2,
					   const CalInt8Vector &pattern,
                                           CalReal32Vector &pha,    // out
				           
                                           const CalInt16Vector &energye3,
					   const CalInt16Vector &energye4,
					   const CalInt8Vector &peripix,
					   const CalInt32Vector &flag
                                           ) 
{
	PROC

        bool withLocbkg = false ;
        bool fulldata = true ;

        // Dummy argument
        CalReal32Vector locbkg;

        AlgoCombine( energye1,energye2,pattern, pha, locbkg, energye3,
                     energye4, peripix, flag, withLocbkg, fulldata ); 
    
	RETURN pha;
} // EnergyCombinator::combine()


CalReal32Vector &EnergyCombinator::combine(const CalReal32Vector &energye1,
					   const CalInt16Vector &energye2,
					   const CalInt8Vector &pattern,
                                           CalReal32Vector &pha,    // out
				           
                                           const CalReal32Vector &locbkg, 
                                           const CalInt16Vector &energye3,
					   const CalInt16Vector &energye4,
					   const CalInt8Vector &peripix,
					   const CalInt32Vector &flag
                                           ) 
{
	PROC

        bool withLocbkg = true ;
        bool fulldata = true ;
        
        AlgoCombine( energye1,energye2,pattern, pha, locbkg, energye3
                    , energye4, peripix, flag, withLocbkg, fulldata );
   
	RETURN pha;
} // EnergyCombinator::combine()


void EnergyCombinator::AlgoCombine( const CalReal32Vector &energye1,
				    const CalInt16Vector &energye2,
				    const CalInt8Vector &pattern,
				    CalReal32Vector &pha,    // out

				    const CalReal32Vector &locbkg, 
				    const CalInt16Vector &energye3,
				    const CalInt16Vector &energye4,
				    const CalInt8Vector &peripix,
				    const CalInt32Vector &flag,
				    bool withLocbkg,
				    bool fulldata    )

{

  //
  // Here is the algorithm which was formerly
  // in the F90 subroutine CAL_mosPhaBuild()
  //
 
  unsigned N = energye1.size();       

  // Zero-order estimate for PHA
  for (unsigned n=0; n<N; ++n)
    pha[n]  = _e1Coefs[pattern[n]]*energye1[n] + 
              _e2Coefs[pattern[n]]*FLT(energye2[n]) ;
      

  if( withLocbkg || fulldata ) {   
   
   // Set the number of pixels >= threshold in the inner corona
   // (ie central 3*3 matrix without the central pixel ) 

   // load the patterns from the CCF
    PatternLibServer *pattLib = calServer.getAtom(pattLib); 

    unsigned nbpat =  pattLib->patterns() ;
    unsigned pixelInE2[ nbpat ] ;

    for (unsigned npat=0; npat<nbpat; ++npat) {
    // get the pattern number 'npat'
      const EventPattern &p = pattLib->pattern(npat); 
      pixelInE2[ npat ] = 0 ;
      for (unsigned n=1; n<4; ++n) {
	for (unsigned m=1; m<4; ++m)
	  {
	    if( (n==2)&&(m==2) ) continue ; // skip central pixel 
	    if( p[n][m] == 1 ) pixelInE2[ npat ]++ ;
	  }
      }
    }


   // Allocate the weight array
    real32 *wght = new real32[ N ] ;

    for (unsigned n=0; n<N; ++n)
      wght[n]  = _e1Coefs[pattern[n]] + 
                 _e2Coefs[pattern[n]]*pixelInE2[pattern[n]] ;

    real32 *locbkg2 ;
    if( withLocbkg ) {
      locbkg2 = &locbkg[0] ;
    } else {
      locbkg2 = new real32[ N ] ; 
      for (unsigned n=0; n<N; ++n) { locbkg2[n] = 0. ; }// init.
    } 

   // locbkg is subtracted once for E1, n times for E2, 8-n times for E3
   // The same weights are applied to E4/(16-peripix) if alpha4 < 0
   // Otherwise E4 is treated as E1, E2 or E3, with weight 16-peripix
   // Do not use E3 or E4 if next to bad line/column
   // Do not use E4 if peripix = 7

    if( fulldata ){ 
  
      for (unsigned n=0; n<N; ++n) {

	bool closebadcolorrow = 
	  flag[n] & ( (1<<EventAttributes::CLOSE_TO_BADCOL) | 
		      (1<<EventAttributes::CLOSE_TO_BADROW)  ) ;
	  
	if ( !closebadcolorrow ) {
	  pha[n] += _e3Coefs[pattern[n]]*FLT(energye3[n]) ; 
	  wght[n] += _e3Coefs[pattern[n]]*FLT(8-pixelInE2[pattern[n]]);
	}

	if (_e4Coefs[pattern[n]] > 0) {   
	  if( (peripix[n] < 7) && !closebadcolorrow ) {
	    pha[n] += _e4Coefs[pattern[n]]*FLT(energye4[n]) ;
	    wght[n] += _e4Coefs[pattern[n]]*FLT(16-peripix[n]);
	  }
	} else {
	  if( (peripix[n] < 7) && !closebadcolorrow ) {
	    locbkg2[n] = (1.+_e4Coefs[pattern[n]])*locbkg2[n] - 
                     _e4Coefs[pattern[n]]*FLT(energye4[n])/FLT(16-peripix[n]) ;
	  }
	    
	}// endelse

      }//endfor

    }// end if fulldata     

    for (unsigned n=0; n<N; ++n)  pha[n] -= locbkg2[n] * wght[n] ;

    if( !withLocbkg ) delete [] locbkg2 ;      
    delete [] wght ;  

  } // end of if ( fulldata || withLocbkg)

} // EnergyCombinator::AlgoCombine()
